/*-----------------------------------------------------------------------

File  : che_learning.h

Author: Stephan Schulz

Contents
 
  Evaluation of a clause by tsm-based learning algorithms

  Copyright 1998, 1999 by the author.
  This code is released under the GNU General Public Licence.
  See the file COPYING in the main CLIB directory for details.
  Run "eprover -h" for contact information.

Changes

<1> Mon Aug 30 19:17:53 MET DST 1999
    New

-----------------------------------------------------------------------*/

#ifndef CHE_LEARNING

#define CHE_LEARNING

#include <che_wfcb.h>
#include <cle_tsmio.h>

/*---------------------------------------------------------------------*/
/*                    Data type declarations                           */
/*---------------------------------------------------------------------*/

#define DEFAULT_POS_MULT 1

typedef struct tsmparamcell
{
   long           vweight;
   long           fweight;
   double         learnweight;
   double         eval_base;
   double         eval_scale;
   bool           flat_clauses;
   TB_p           bank;
   PatternSubst_p pat_subst;
   TSMAdmin_p     tsmadmin;
}TSMParamCell, *TSMParam_p;


/*---------------------------------------------------------------------*/
/*                Exported Functions and Variables                     */
/*---------------------------------------------------------------------*/


#define TSMParamCellAlloc() (TSMParamCell*) \
   SizeMalloc(sizeof(TSMParamCell))
#define TSMParamCellFree(junk) \
   SizeFree(junk, sizeof(TSMParamCell))
   
   
WFCB_p TSMWeightInit(ClausePrioFun prio_fun, int fweight,
		     int vweight, bool flat_clauses, double
		     learnweight, char* kb, ProofState_p state, long
		     sel_no, double set_part, double dist_part,
		     IndexType indextype, TSMType tsmtype, long depth,
		     double proofs_w, double dist_w,  double p_simp_w,
		     double f_simp_w, double p_gen_w, double f_gen_w,
		     double subsum_w);

WFCB_p TSMWeightParse(Scanner_p in, OCB_p ocb, ProofState_p state);

double TSMWeightCompute(void* data, Clause_p clause);

void   TSMWeightExit(void* data);

#endif

/*---------------------------------------------------------------------*/
/*                        End of File                                  */
/*---------------------------------------------------------------------*/





