/*-----------------------------------------------------------------------

File  : che_litselection.h

Author: Stephan Schulz

Contents

  Functions for selection certain literals (and hence superposition
  strategies).

  Copyright 1998, 1999 by the author.
  This code is released under the GNU General Public Licence.
  See the file COPYING in the main CLIB directory for details.
  Run "eprover -h" for contact information.

Changes

<1> Fri May 21 22:17:06 GMT 1999
    New

-----------------------------------------------------------------------*/

#ifndef CHE_LITSELECTION

#define CHE_LITSELECTION

#include <clb_simple_stuff.h>
#include <ccl_clauses.h>

/*---------------------------------------------------------------------*/
/*                    Data type declarations                           */
/*---------------------------------------------------------------------*/

typedef void (*LiteralSelectionFun)(Clause_p clause);

/*---------------------------------------------------------------------*/
/*                Exported Functions and Variables                     */
/*---------------------------------------------------------------------*/

extern char* LiteralSelectionFunNames[];

LiteralSelectionFun GetLitSelFun(char* name);
char*               GetLitSelName(LiteralSelectionFun fun);

void SelectNoLiterals(Clause_p clause);
void SelectNoGeneration(Clause_p clause);
void SelectNegativeLiterals(Clause_p clause);
void PSelectNegativeLiterals(Clause_p clause);
void SelectFirstVariableLiteral(Clause_p clause);
void PSelectFirstVariableLiteral(Clause_p clause);
void SelectLargestNegativeLiteral(Clause_p clause);
void PSelectLargestNegativeLiteral(Clause_p clause);
void SelectSmallestNegativeLiteral(Clause_p clause);
void PSelectSmallestNegativeLiteral(Clause_p clause);
void SelectDiffNegativeLiteral(Clause_p clause);
void PSelectDiffNegativeLiteral(Clause_p clause);
void SelectGroundNegativeLiteral(Clause_p clause);
void PSelectGroundNegativeLiteral(Clause_p clause);
void SelectOptimalLiteral(Clause_p clause);
void PSelectOptimalLiteral(Clause_p clause);
void SelectCondOptimalLiteral(Clause_p clause);
void PSelectCondOptimalLiteral(Clause_p clause);
void SelectAllCondOptimalLiteral(Clause_p clause);
void PSelectAllCondOptimalLiteral(Clause_p clause);
void SelectDepth2OptimalLiteral(Clause_p clause);
void PSelectDepth2OptimalLiteral(Clause_p clause);
void SelectPDepth2OptimalLiteral(Clause_p clause);
void PSelectPDepth2OptimalLiteral(Clause_p clause);
void SelectNDepth2OptimalLiteral(Clause_p clause);
void PSelectNDepth2OptimalLiteral(Clause_p clause);
void SelectNonRROptimalLiteral(Clause_p clause);
void PSelectNonRROptimalLiteral(Clause_p clause);
void SelectNonStrongRROptimalLiteral(Clause_p clause);
void PSelectNonStrongRROptimalLiteral(Clause_p clause);

void SelectAntiRROptimalLiteral(Clause_p clause);
void PSelectAntiRROptimalLiteral(Clause_p clause);

void SelectNonAntiRROptimalLiteral(Clause_p clause);
void PSelectNonAntiRROptimalLiteral(Clause_p clause);

void SelectStrongRRNonRROptimalLiteral(Clause_p clause);
void PSelectStrongRRNonRROptimalLiteral(Clause_p clause);

#endif

/*---------------------------------------------------------------------*/
/*                        End of File                                  */
/*---------------------------------------------------------------------*/





