/*-----------------------------------------------------------------------

File  : cle_tsmio.c

Author: Stephan Schulz

Contents
 

  Copyright 1998, 1999 by the author.
  This code is released under the GNU General Public Licence.
  See the file COPYING in the main CLIB directory for details.
  Run "eprover -h" for contact information.

Changes

<1> Tue Aug 31 13:47:43 MET DST 1999
    New

-----------------------------------------------------------------------*/

#include "cle_tsmio.h"



/*---------------------------------------------------------------------*/
/*                        Global Variables                             */
/*---------------------------------------------------------------------*/

static double selection_weights[] = SEL_FEATURE_WEIGHTS;


/*---------------------------------------------------------------------*/
/*                      Forward Declarations                           */
/*---------------------------------------------------------------------*/


/*---------------------------------------------------------------------*/
/*                         Internal Functions                          */
/*---------------------------------------------------------------------*/



/*---------------------------------------------------------------------*/
/*                         Exported Functions                          */
/*---------------------------------------------------------------------*/


/*-----------------------------------------------------------------------
//
// Function: ExampleSetPrepare()
//
//   Create a flat annotated example set with examples tailored to
//   target from annoset.
//
// Global Variables: -
//
// Side Effects    : Changes annoset and flatset.
//
/----------------------------------------------------------------------*/

long ExampleSetPrepare(FlatAnnoSet_p flatset, AnnoSet_p annoset, double
		       evalweights[], ExampleSet_p examples,  Sig_p sig,
		       ClauseSet_p target, long sel_no, double set_part,
		       double dist_part)
{
   Features_p   targetfeatures = FeaturesAlloc(); 
   PStack_p     example_ids = PStackAlloc();
   long         res;
   
   ComputeClauseSetNumFeatures(targetfeatures, target, sig);
   ExampleSetSelectByDist(example_ids, examples, targetfeatures,
			  SEL_PRED_WEIGHT, SEL_FUNC_WEIGHT,
			  selection_weights, sel_no, set_part,
			  dist_part);
   AnnoSetFlatten(annoset, example_ids);   
   AnnoSetNormalizeFlatAnnos(annoset);
   res = FlatAnnoSetTranslate(flatset, annoset, evalweights);   
   PStackFree(example_ids);
   FeaturesFree(targetfeatures);
   return res;
}


/*-----------------------------------------------------------------------
//
// Function: ExampleSetFromKB()
//
//   Create a flat annotated example set from a knowledge base.
//
// Global Variables: -
//
// Side Effects    : I/O, lot's of memory operations, changes
//                   sig.
//
/----------------------------------------------------------------------*/

void ExampleSetFromKB(AnnoSet_p annoset, FlatAnnoSet_p flatset, bool
		      flat_patterns, TB_p bank, double evalweights[],
		      char* kb, Sig_p sig, ClauseSet_p target, long
		      sel_no, double set_part, double dist_part)
{
   DStr_p         filename = DStrAlloc();
   ExampleSet_p   proofexamples;
   Scanner_p      in;
   
   proofexamples = ExampleSetAlloc();

   in = CreateScanner(StreamTypeFile,
		      KBFileName(filename, kb, "signature"),
		      true, NULL);
   SigParse(in, sig, special);
   DestroyScanner(in);

   in = CreateScanner(StreamTypeFile,
		      KBFileName(filename, kb, "problems"),
		      true, NULL);
   ExampleSetParse(in, proofexamples);
   DestroyScanner(in);
   DStrFree(filename);
   
   if(flat_patterns)
   {
      AnnoSetRecToFlatEnc(bank, annoset);
   }
   ExampleSetPrepare(flatset, annoset, evalweights, proofexamples,
		     sig, target, sel_no, set_part, dist_part);
   ExampleSetFree(proofexamples);
}


/*-----------------------------------------------------------------------
//
// Function: TSMFromKB()
//
//   Create a tsm for evaluating clauses for proving target from kb
//
// Global Variables: 
//
// Side Effects    : 
//
/----------------------------------------------------------------------*/

TSMAdmin_p TSMFromKB(bool flat_patterns, double evalweights[], char*
		     kb,  Sig_p sig, ClauseSet_p target, long sel_no,
		     double set_part, double dist_part, IndexType
		     indextype, TSMType tsmtype, long indexdepth)
{
   DStr_p         filename = DStrAlloc();
   AnnoSet_p      annoset;
   FlatAnnoSet_p  flatset = FlatAnnoSetAlloc();
   TB_p bank =    TBAlloc(TPIgnoreProps, sig);
   TSMAdmin_p     admin;
   PatternSubst_p subst;
   Scanner_p      in;
   
   in = CreateScanner(StreamTypeFile,
		      KBFileName(filename, kb, "clausepatterns"),
		      true, NULL);
   annoset = AnnoSetParse(in, bank, KB_ANNOTATION_NO);
   DestroyScanner(in);
   DStrFree(filename);

   ExampleSetFromKB(annoset, flatset, flat_patterns, bank,
		    evalweights, kb, sig, target, sel_no, set_part,
		    dist_part);
   
   subst = PatternDefaultSubstAlloc(sig);
   AnnoSetComputePatternSubst(subst, annoset);
   admin = TSMAdminAlloc(sig, tsmtype);
   TSMAdminBuildTSM(admin, flatset, indextype, indexdepth, subst);
   
   FlatAnnoSetFree(flatset);
   bank->sig = NULL;
   AnnoSetFree(annoset);   
   TBFree(bank); 
   VERBOUT("TSM created\n");
   return admin;
}



/*---------------------------------------------------------------------*/
/*                        End of File                                  */
/*---------------------------------------------------------------------*/


